# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"read_csv": "20_data.ipynb",
         "concat": "20_data.ipynb",
         "add_tables": "22_statistics.ipynb",
         "prepare_labrie_data": "20_data.ipynb",
         "prepare_braverman_data": "20_data.ipynb",
         "prepare_philander_data": "20_data.ipynb",
         "generate_bets": "20_data.ipynb",
         "check_data": "20_data.ipynb",
         "summarise_app": "20_data.ipynb",
         "summarise_providers": "20_data.ipynb",
         "plot_player_career": "20_data.ipynb",
         "plot_player_career_split": "20_data.ipynb",
         "plot_provider_dates": "20_data.ipynb",
         "plot_provider_bets": "20_data.ipynb",
         "create_measures_table": "21_measures.ipynb",
         "duration": "21_measures.ipynb",
         "frequency": "21_measures.ipynb",
         "frequency_aggregated": "21_measures.ipynb",
         "frequency_percent": "21_measures.ipynb",
         "number_of_bets": "21_measures.ipynb",
         "mean_bets_per_day_aggregated": "21_measures.ipynb",
         "mean_bets_per_day": "21_measures.ipynb",
         "median_bets_per_day": "21_measures.ipynb",
         "num_sessions": "21_measures.ipynb",
         "total_play_time": "21_measures.ipynb",
         "mean_sessions_per_day": "21_measures.ipynb",
         "median_sessions_per_day": "21_measures.ipynb",
         "mean_session_duration": "21_measures.ipynb",
         "median_session_duration": "21_measures.ipynb",
         "mean_play_time": "21_measures.ipynb",
         "median_play_time": "21_measures.ipynb",
         "num_bets_deviation_per_day": "21_measures.ipynb",
         "inactive_day_streak_variance": "21_measures.ipynb",
         "persistence": "21_measures.ipynb",
         "bet_count_trajectory": "21_measures.ipynb",
         "mean_bets_per_session": "21_measures.ipynb",
         "median_bets_per_session": "21_measures.ipynb",
         "bets_per_session_variance": "21_measures.ipynb",
         "session_duration_variance": "21_measures.ipynb",
         "session_duration_trajectory": "21_measures.ipynb",
         "sawtooth_occurances": "21_measures.ipynb",
         "active_day_trajectory": "21_measures.ipynb",
         "total_wagered": "21_measures.ipynb",
         "mean_bet_size": "21_measures.ipynb",
         "median_bet_size": "21_measures.ipynb",
         "max_bet": "21_measures.ipynb",
         "min_bet": "21_measures.ipynb",
         "bet_size_range": "21_measures.ipynb",
         "bet_size_deviation": "21_measures.ipynb",
         "trajectory": "21_measures.ipynb",
         "mean_amount_wagered_per_day": "21_measures.ipynb",
         "median_amount_wagered_per_day": "21_measures.ipynb",
         "amount_wagered_per_day_variance": "21_measures.ipynb",
         "mean_amount_wagered_per_session": "21_measures.ipynb",
         "median_amount_wagered_per_session": "21_measures.ipynb",
         "total_amount_wagered_across_duration": "21_measures.ipynb",
         "amount_wagered_per_session_variance": "21_measures.ipynb",
         "bet_size_trajectory": "21_measures.ipynb",
         "net_loss": "21_measures.ipynb",
         "percent_loss": "21_measures.ipynb",
         "payout_bet_count_ratio": "21_measures.ipynb",
         "payout_bet_size_ratio": "21_measures.ipynb",
         "mean_net_loss_per_session": "21_measures.ipynb",
         "median_net_loss_per_session": "21_measures.ipynb",
         "sum_of_payouts": "21_measures.ipynb",
         "relative_big_win": "21_measures.ipynb",
         "big_win": "21_measures.ipynb",
         "mean_payout_size": "21_measures.ipynb",
         "median_payout_size": "21_measures.ipynb",
         "overall_binary_loser": "21_measures.ipynb",
         "clamped_net_win": "21_measures.ipynb",
         "net_loss_on_last_day": "21_measures.ipynb",
         "mean_loss_per_bet": "21_measures.ipynb",
         "median_loss_per_bet": "21_measures.ipynb",
         "theoretical_loss": "21_measures.ipynb",
         "mean_odds_per_bet": "21_measures.ipynb",
         "median_odds_per_bet": "21_measures.ipynb",
         "num_providers_used": "21_measures.ipynb",
         "mean_game_types_per_day": "21_measures.ipynb",
         "calculate_labrie_measures": "21_measures.ipynb",
         "calculate_braverman_measures": "21_measures.ipynb",
         "available_measures": "21_measures.ipynb",
         "check_measure_data": "21_measures.ipynb",
         "get_sessions": "21_measures.ipynb",
         "get_daily_bets": "21_measures.ipynb",
         "standardise_measures_table": "21_measures.ipynb",
         "split_measures_table": "21_measures.ipynb",
         "generic_trajectory": "21_measures.ipynb",
         "plot_individual": "21_measures.ipynb",
         "plot_continuous_time_domain": "21_measures.ipynb",
         "discretise_player_bets": "21_measures.ipynb",
         "plot_discrete_time_domain": "21_measures.ipynb",
         "plot_session_domain": "21_measures.ipynb",
         "plot_cost_domain": "21_measures.ipynb",
         "show_domain_differences": "21_measures.ipynb",
         "descriptive_table": "22_statistics.ipynb",
         "ks_test": "22_statistics.ipynb",
         "cohens_d": "22_statistics.ipynb",
         "spearmans_r": "22_statistics.ipynb",
         "calculate_walker_matrix": "22_statistics.ipynb",
         "label_overlap_table": "22_statistics.ipynb",
         "color_matrix": "22_statistics.ipynb",
         "top_split": "23_labelling.ipynb",
         "get_labelled_groups": "23_labelling.ipynb",
         "spending_portions": "23_labelling.ipynb",
         "k_means": "24_machine_learning.ipynb",
         "k_means_range": "24_machine_learning.ipynb",
         "k_means_ensemble": "24_machine_learning.ipynb",
         "agglomerative_cluster": "24_machine_learning.ipynb",
         "describe_clusters": "24_machine_learning.ipynb",
         "neural_network_classification": "24_machine_learning.ipynb",
         "neural_network_regression": "24_machine_learning.ipynb",
         "logistic_regression": "24_machine_learning.ipynb",
         "lasso_logistic_regression": "24_machine_learning.ipynb",
         "svm_eps_regression": "24_machine_learning.ipynb",
         "svm_c_classification": "24_machine_learning.ipynb",
         "svm_one_classification": "24_machine_learning.ipynb",
         "rf_regression": "24_machine_learning.ipynb",
         "rf_classification": "24_machine_learning.ipynb",
         "compute_performance": "24_machine_learning.ipynb",
         "plot_cluster_sizes": "24_machine_learning.ipynb",
         "plot_agglomeration_dendrogram": "24_machine_learning.ipynb"}

modules = ["data.py",
           "measures.py",
           "statistics.py",
           "labelling.py",
           "machine_learning.py",
           "__init__.py"]

doc_url = "https://gamba-dev.github.io/gamba/"

git_url = "https://github.com/gamba-dev/gamba/tree/main/"

def custom_doc_links(name): return None
